#!/usr/bin/env ruby
require 'optparse'
require 'pathname'
require 'erb'
=begin
CoC

data dir = /home/user/data/NAME/
web dir  = /home/user/public_html/NAME/
svn dir  = /home/user/svn/NAME/

=end

SERVER_URL = Pathname.new("http://mono.kmc.gr.jp/")
PUBLIC_HTML = "public_html"
HIKI_INSTALL_PATH = "/home/yhara/bin/hiki"
DRY_RUN = false

def abort(msg)
  puts msg + " abort."
  #exit
end

def shell(cmd)
  puts cmd
  system cmd unless DRY_RUN
end

def make_directory(path, opt={})
  if path.exist? and opt[:must_not_exist]
    abort "directory #{path} already exists."
  end
  unless path.directory?
    puts "making directory #{path}"
    path.mkdir unless DRY_RUN
  end
end

def make_file(path, data, perm)
  if path.exist? 
    abort "file #{path} already exists."
  end  
  puts "writing file #{path} with permission #{perm}"
  unless DRY_RUN
    if perm
      path.open("wb", perm){|f| f.write data} 
    else
      path.open("wb", perm){|f| f.write data} 
    end
  end
end

def run_erb(filename, args)
  template = File.read(File.join(File.dirname(__FILE__), filename))
  ERB.new(template).result(binding)
end

if ARGV.size != 1
  puts "usage: #{$0} WIKINAME"
else
  name = ARGV.first
  user = ENV['USER']

  home = Pathname.new("/home")
  make_directory(home + user + "data")
  make_directory(home + user + PUBLIC_HTML)
  make_directory(home + user + "svn")

  data_dir = home + user + "data" + name
  make_directory(data_dir, {:must_not_exist => true})
  web_dir  = home + user + PUBLIC_HTML + name
  make_directory(web_dir, {:must_not_exist => true})
  svn_dir  = home + user + "svn" + name
  make_directory(svn_dir, {:must_not_exist => true})

  puts "---- init web dir ----"
  opts = [ 
    { 
      :tmpl => "hiki.cgi.erb",
      :args => {:hiki_dir => HIKI_INSTALL_PATH},
      :path => web_dir + "hiki.cgi", 
      :perm => 0775
    },
    { 
      :tmpl => "hikiconf.rb.erb", 
      :args => {:repos_root => home + user + "svn" + name,
                :data_path => home + user + "data" + name},
      :path => web_dir + "hikiconf.rb" 
    },
    {
      :tmpl => "htaccess.erb",
      :args => {},
      :path => web_dir + ".htaccess",
    },
  ]
  opts.each do |opt|
    make_file(opt[:path], run_erb(opt[:tmpl], opt[:args]), opt[:perm])
  end

  puts "---- init data dir ----"
  puts "access #{SERVER_URL + "~#{user}" + "#{name}/"} to init data dir"
  puts "(press enter if done)"
  $stdin.gets

  puts "---- init svn dir ----"
  shell "svnadmin create #{svn_dir}"
  text_dir = data_dir + 'text'
  shell "svn import -m 'import' #{text_dir} file://#{svn_dir}"
  shell "rm -ri #{text_dir}"
  shell "svn co file://#{svn_dir} #{text_dir}"

  puts "---- done. ----"
end
 
